#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.add_counter_formula_input import AddCounterFormulaInput


# fmt: off
QUERY: List[str] = ["""
mutation addCounterFormula($input: AddCounterFormulaInput!) {
  addCounterFormula(input: $input) {
    id
    mandatory
    formulaFk{id}
    counterFk{id}
    
  }
}

"""
]


class addCounterFormula:
    @dataclass(frozen=True)
    class addCounterFormulaData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class CounterFormula(DataClassJsonMixin):
            @dataclass(frozen=True)
            class Formula(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Counter(DataClassJsonMixin):
                id: str

            id: str
            mandatory: bool
            formulaFk: Formula
            counterFk: Counter

        addCounterFormula: CounterFormula

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: AddCounterFormulaInput) -> addCounterFormulaData.CounterFormula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addCounterFormulaData.from_dict(response_text)
        return res.addCounterFormula

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: AddCounterFormulaInput) -> addCounterFormulaData.CounterFormula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.addCounterFormulaData.from_dict(response_text)
        return res.addCounterFormula



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.tech_filter_input import TechFilterInput
from ..input.tech_order import TechOrder


# fmt: off
QUERY: List[str] = ["""
query techs(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: TechOrder
$filterBy: [TechFilterInput!]
){
techs(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
id
name

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class techs:
    @dataclass(frozen=True)
    class techsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class TechConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class TechEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Tech(DataClassJsonMixin):
                    id: str
                    name: str

                node: Optional[Tech]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[TechEdge]
            pageInfo: PageInfo

        techs: TechConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[TechOrder] = None, filterBy: List[TechFilterInput] = []) -> techsData.TechConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.techsData.from_dict(response_text)
        return res.techs

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[TechOrder] = None, filterBy: List[TechFilterInput] = []) -> techsData.TechConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.techsData.from_dict(response_text)
        return res.techs



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Iterator, List, Mapping, Optional

from psym.client import SymphonyClient
from psym.common.cache import EQUIPMENT_TYPES, PORT_TYPES
from psym.common.data_class import (
    EquipmentPortDefinition,
    EquipmentType,
    PropertyDefinition,
    PropertyValue,
)
from psym.common.data_enum import Entity
from psym.common.data_format import (
    format_to_equipment,
    format_to_equipment_type,
    format_to_property_type_input,
    format_to_property_type_inputs,
)

from .._utils import (
    get_graphql_property_type_inputs,
    get_port_definition_input,
    get_position_definition_input,
)
from ..exceptions import EntityNotFoundError
from ..graphql.input.add_equipment_type_input import AddEquipmentTypeInput
from ..graphql.input.edit_equipment_type_input import EditEquipmentTypeInput
from ..graphql.input.equipment_port_input import EquipmentPortInput
from ..graphql.input.equipment_position_input import EquipmentPositionInput
from ..graphql.input.property_type_input import PropertyTypeInput
from ..graphql.mutation.add_equipment_type import AddEquipmentTypeMutation
from ..graphql.mutation.edit_equipment_type import EditEquipmentTypeMutation
from ..graphql.mutation.remove_equipment_type import RemoveEquipmentTypeMutation
from ..graphql.query.equipment_type_details import EquipmentTypeDetailsQuery
from ..graphql.query.equipment_type_equipments import EquipmentTypeEquipmentQuery
from ..graphql.query.equipment_types import EquipmentTypesQuery
from ..graphql.input.equipment_port_connection_input import EquipmentPortConnectionInput
from .equipment import delete_equipment
from .property_type import (
    edit_property_type,
    get_property_type,
    get_property_type_by_external_id,
)


def _populate_equipment_types(client: SymphonyClient) -> None:
    edges = EquipmentTypesQuery.execute(client).edges

    for edge in edges:
        node = edge.node
        if node:
            EQUIPMENT_TYPES[node.name] = format_to_equipment_type(
                equipment_type_fragment=node
            )


def _add_equipment_type(
    client: SymphonyClient,
    name: str,
    category: Optional[str],
    properties: List[PropertyTypeInput],
    position_definitions: List[EquipmentPositionInput],
    port_definitions: List[EquipmentPortInput],
) -> AddEquipmentTypeMutation.AddEquipmentTypeMutationData.EquipmentType:
    return AddEquipmentTypeMutation.execute(
        client,
        AddEquipmentTypeInput(
            name=name,
            category=category,
            positions=position_definitions,
            ports=port_definitions,
            properties=properties,
        ),
    )


def get_or_create_equipment_type(
    client: SymphonyClient,
    name: str,
    category: str,
    properties: List[PropertyDefinition],
    port_definitions: List[EquipmentPortDefinition],
    position_list: List[str],
) -> EquipmentType:
    """This function checks equipment type existence,
    in case it is not found, creates one.

    :param name: Equipment type name
    :type name: str
    :param category: Category name
    :type category: str
    :param properties: List of property definitions
    :type properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param port_definitions: EquipmentPortDefinitions list
    :type port_definitions: List[ :class:`~psym.common.data_class.EquipmentPortDefinition` ]
    :param position_list: List of positions names
    :type position_list: List[str]

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        e_type = client.get_or_create_equipment_type(
            name="Tp-Link T1600G",
            category="Router",
            properties=[
                PropertyDefinition(
                    property_name="IP",
                    property_kind=PropertyKind.string,
                    default_raw_value=None,
                    is_fixed=True
                )
            ],
            port_definitions=[
                EquipmentPortDefinition(
                    name="tp_link_port",
                    visible_label="TP-Link port",
                    port_definition_index=0,
                    port_type_name="port type 1",
                )
            ],
            position_list=[],
        )
    """
    if name in EQUIPMENT_TYPES:
        return EQUIPMENT_TYPES[name]
    return add_equipment_type(
        client, name, category, properties, port_definitions, position_list
    )


def _edit_equipment_type(
    client: SymphonyClient,
    equipment_type_id: str,
    name: str,
    category: Optional[str],
    properties: List[PropertyTypeInput],
    position_definitions: List[EquipmentPositionInput],
    port_definitions: List[EquipmentPortInput],
) -> EditEquipmentTypeMutation.EditEquipmentTypeMutationData.EquipmentType:
    return EditEquipmentTypeMutation.execute(
        client,
        EditEquipmentTypeInput(
            id=equipment_type_id,
            name=name,
            category=category,
            positions=position_definitions,
            ports=port_definitions,
            properties=properties,
        ),
    )


def _update_equipment_type(
    client: SymphonyClient,
    equipment_type_id: str,
    name: str,
    category: Optional[str],
    properties: List[PropertyTypeInput],
    position_definitions: List[EquipmentPositionInput],
    port_definitions: List[EquipmentPortInput],
) -> EquipmentType:

    equipment_type_result = _edit_equipment_type(
        client=client,
        equipment_type_id=equipment_type_id,
        name=name,
        category=category,
        properties=properties,
        position_definitions=position_definitions,
        port_definitions=port_definitions,
    )
    equipment_type = format_to_equipment_type(
        equipment_type_fragment=equipment_type_result
    )
    EQUIPMENT_TYPES[name] = equipment_type
    return equipment_type


def add_equipment_type(
    client: SymphonyClient,
    name: str,
    category: str,
    properties: List[PropertyDefinition],
    port_definitions: List[EquipmentPortDefinition],
    position_list: List[str],
) -> EquipmentType:
    """This function creates new equipment type.

    :param name: Equipment type name
    :type name: str
    :param category: Category name
    :type category: str
    :param properties: List of property definitions
    :type properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param port_definitions: EquipmentPortDefinitions list
    :type port_definitions: List[ :class:`~psym.common.data_class.EquipmentPortDefinition` ]
    :param position_list: List of positions names
    :type position_list: List[str]

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        e_type = client.add_equipment_type(
            name="Tp-Link T1600G",
            category="Router",
            properties=[
                PropertyDefinition(
                    property_name="IP",
                    property_kind=PropertyKind.string,
                    default_raw_value=None,
                    is_fixed=True
                )
            ],
            port_definitions=[
                EquipmentPortDefinition(
                    name="tp_link_port",
                    visible_label="TP-Link port",
                    port_definition_index=0,
                    port_type_name="port type 1",
                )
            ],
            position_list=[],
        )
    """
    new_property_types = format_to_property_type_inputs(data=properties)
    new_port_definitions = [
        EquipmentPortInput(
            name=pd.name,
            index=pd.port_definition_index,
            visibleLabel=pd.visible_label,
            portTypeID=PORT_TYPES[pd.port_type_name].id if pd.port_type_name else None,
            connectedPorts=[
                EquipmentPortConnectionInput(id=cp.id, name=cp.name)
                for cp in pd.connected_ports
            ],
        )
        for pd in port_definitions
    ]
    position_definitions = [
        EquipmentPositionInput(name=position) for position in position_list
    ]
    equipment_type_result = _add_equipment_type(
        client,
        name,
        category,
        new_property_types,
        position_definitions,
        new_port_definitions,
    )
    equipment_type = format_to_equipment_type(
        equipment_type_fragment=equipment_type_result
    )
    EQUIPMENT_TYPES[name] = equipment_type
    return equipment_type


def add_property_types_to_equipment_type(
    client: SymphonyClient,
    equipment_type_id: str,
    new_properties: List[PropertyDefinition],
) -> EquipmentType:
    """This function adds new property types to existing equipment type.

    :param equipment_type_id: Existing equipment type ID
    :type equipment_type_id: str
    :param new_properties: List of property definitions
    :type new_properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ]

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        equipment_type = client.add_property_types_to_equipment_type(
            equipment_type_id="12345678",
            new_properties=[
                PropertyDefinition(
                    property_name="Contact",
                    property_kind=PropertyKind.string,
                    default_raw_value=None,
                    is_fixed=True
                )
            ],
        )
    """
    equipment_type = get_equipment_type_by_id(
        client=client, equipment_type_id=equipment_type_id
    )
    new_property_type_inputs = format_to_property_type_inputs(data=new_properties)

    position_definitions = [
        get_position_definition_input(position_definition, is_new=False)
        for position_definition in equipment_type.position_definitions
    ]
    port_definitions = [
        get_port_definition_input(port_definition, is_new=False)
        for port_definition in equipment_type.port_definitions
    ]

    return _update_equipment_type(
        client=client,
        equipment_type_id=equipment_type.id,
        name=equipment_type.name,
        category=equipment_type.category,
        properties=new_property_type_inputs,
        position_definitions=position_definitions,
        port_definitions=port_definitions,
    )


def edit_equipment_type(
    client: SymphonyClient,
    name: str,
    new_positions_list: List[str],
    new_port_definitions: List[EquipmentPortDefinition],
    new_properties: Optional[Mapping[str, PropertyValue]] = None,
) -> EquipmentType:
    """Edit existing equipment type.

    :param name: Equipment type name
    :type name: str
    :param new_positions_list: List of new positions
    :type new_positions_list: List[str]
    :param new_port_definitions: EquipmentPortDefinitions list
    :type new_port_definitions: List[ :class:`~psym.common.data_class.EquipmentPortDefinition` ]
    :param new_properties: Property definitions list
    :type new_properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ], optional

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        edited_equipment = client.edit_equipment_type(
            name="Card",
            new_positions_list=[],
            new_port_definitions=[
                EquipmentPortDefinition(
                    name="tp_link_port",
                    visible_label="TP-Link port",
                    port_definition_index=0,
                    port_type_name="port type 1",
                )
            ],
        )
    """
    equipment_type = EQUIPMENT_TYPES[name]
    position_definitions = [
        get_position_definition_input(position_definition, is_new=False)
        for position_definition in equipment_type.position_definitions
    ] + [EquipmentPositionInput(name=position) for position in new_positions_list]
    port_definitions = [
        get_port_definition_input(port_definition, is_new=False)
        for port_definition in equipment_type.port_definitions
    ] + [
        EquipmentPortInput(
            name=pd.name,
            index=pd.port_definition_index,
            visibleLabel=pd.visible_label,
            portTypeID=PORT_TYPES[pd.port_type_name].id if pd.port_type_name else None,
            connectedPorts=[
                EquipmentPortConnectionInput(id=cp.id, name=cp.name)
                for cp in pd.connected_ports
            ],
        )
        for pd in new_port_definitions
    ]
    new_property_type_inputs = []
    if new_properties:
        property_types = PORT_TYPES[equipment_type.name].property_types
        new_property_type_inputs = get_graphql_property_type_inputs(
            property_types, new_properties
        )

    return _update_equipment_type(
        client=client,
        equipment_type_id=equipment_type.id,
        name=equipment_type.name,
        category=equipment_type.category,
        properties=new_property_type_inputs,
        position_definitions=position_definitions,
        port_definitions=port_definitions,
    )


def get_equipment_type_by_id(
    client: SymphonyClient, equipment_type_id: str
) -> EquipmentType:
    """This function returns an equipment type.
    It gets the requested equipment type ID

    :param equipment_type_id: Equipment type ID
    :type equipment_type_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Equipment type does not found

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        equipment_type = client.get_equipment_type_by_id(equipment_type_id="12345678)
    """
    result = EquipmentTypeDetailsQuery.execute(client, id=equipment_type_id)
    if not result:
        raise EntityNotFoundError(
            entity=Entity.EquipmentType, entity_id=equipment_type_id
        )

    return format_to_equipment_type(equipment_type_fragment=result)


def get_equipment_types(client: SymphonyClient) -> Iterator[EquipmentType]:
    """Get the iterator of equipment types

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.EquipmentType` ]

    **Example**

    .. code-block:: python

        equipment_types = client.get_equipment_types()
        for equipment_type in equipment_types:
            print(equipment_type.name)
    """
    result = EquipmentTypesQuery.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node is not None:
            yield format_to_equipment_type(equipment_type_fragment=node)


def copy_equipment_type(
    client: SymphonyClient, curr_equipment_type_name: str, new_equipment_type_name: str
) -> EquipmentType:
    """Copy existing equipment type.

    :param curr_equipment_type_name: Existing equipment type name
    :type curr_equipment_type_name: str
    :param new_equipment_type_name: New equipment type name
    :type new_equipment_type_name: str

    :raises:
        FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        e_type = client.copy_equipment_type(
            curr_equipment_type_name="Card",
            new_equipment_type_name="External_Card",
        )
    """
    equipment_type = EQUIPMENT_TYPES[curr_equipment_type_name]

    new_property_types = [
        format_to_property_type_input(property_type)
        for property_type in equipment_type.property_types
    ]

    new_position_definitions = [
        get_position_definition_input(position_definition)
        for position_definition in equipment_type.position_definitions
    ]

    new_port_definitions = [
        get_port_definition_input(port_definition)
        for port_definition in equipment_type.port_definitions
    ]

    equipment_type_result = _add_equipment_type(
        client,
        new_equipment_type_name,
        equipment_type.category,
        new_property_types,
        new_position_definitions,
        new_port_definitions,
    )

    new_equipment_type = format_to_equipment_type(
        equipment_type_fragment=equipment_type_result
    )

    EQUIPMENT_TYPES[new_equipment_type_name] = new_equipment_type
    return new_equipment_type


def get_equipment_type_property_type(
    client: SymphonyClient, equipment_type_name: str, property_type_id: str
) -> PropertyDefinition:
    """Get property type by ID on specific equipment type.

    :param equipment_type_name: Existing equipment type name
    :type equipment_type_name: str
    :param property_type_id: Property type ID
    :type property_type_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: property type with id=`property_type_id` is not found

    :return: PropertyDefinition object
    :rtype: :class:`~psym.common.data_class.PropertyDefinition`

    **Example**

    .. code-block:: python

        property_type = client.get_equipment_type_property_type(
            equipment_type_name="Card",
            property_type_id="12345",
        )
    """
    return get_property_type(
        client=client,
        entity_type=Entity.EquipmentType,
        entity_name=equipment_type_name,
        property_type_id=property_type_id,
    )


def get_equipment_type_property_type_by_external_id(
    client: SymphonyClient, equipment_type_name: str, property_type_external_id: str
) -> PropertyDefinition:
    """Get property type by external ID on specific equipment type.

    :param equipment_type_name: Existing equipment type name
    :type equipment_type_name: str
    :param property_type_external_id: Property type external ID
    :type property_type_external_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: property type with external_id=`property_type_external_id` is not found

    :return: PropertyDefinition object
    :rtype: :class:`~psym.common.data_class.PropertyDefinition`

    **Example**

    .. code-block:: python

        property_type = client.get_equipment_type_property_type_by_external_id(
            equipment_type_name="Card",
            property_type_external_id="12345",
        )
    """
    return get_property_type_by_external_id(
        client=client,
        entity_type=Entity.EquipmentType,
        entity_name=equipment_type_name,
        property_type_external_id=property_type_external_id,
    )


def edit_equipment_type_property_type(
    client: SymphonyClient,
    equipment_type_name: str,
    property_type_id: str,
    new_property_definition: PropertyDefinition,
) -> EquipmentType:
    """Edit specific property type on specific equipment type.


    :param equipment_type_name: Existing equipment type name
    :type equipment_type_name: str
    :param property_type_id: Existing property type ID
    :type property_type_id: str
    :param new_property_definition: New property definition
    :type new_property_definition: :class:`~psym.common.data_class.PropertyDefinition`

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: if property type name is not found
        * FailedOperationException: Internal symphony error

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        e_type = client.edit_equipment_type_property_type(
            equipment_type_name="Card",
            property_type_id="111669149698",
            new_property_definition=PropertyDefinition(
                property_name=property_type_name,
                property_kind=PropertyKind.string,
                default_raw_value=None,
                is_fixed=False,
                external_id="12345",
            ),
        )
    """
    equipment_type = EQUIPMENT_TYPES[equipment_type_name]
    edited_property_types = edit_property_type(
        client=client,
        entity_type=Entity.EquipmentType,
        entity_name=equipment_type_name,
        property_type_id=property_type_id,
        new_property_definition=new_property_definition,
    )
    position_definitions = [
        get_position_definition_input(position_definition, is_new=False)
        for position_definition in equipment_type.position_definitions
        if equipment_type.position_definitions
    ]
    port_definitions = [
        get_port_definition_input(port_definition, is_new=False)
        for port_definition in equipment_type.port_definitions
        if equipment_type.port_definitions
    ]

    return _update_equipment_type(
        client=client,
        equipment_type_id=equipment_type.id,
        name=equipment_type.name,
        category=equipment_type.category,
        properties=edited_property_types,
        position_definitions=position_definitions,
        port_definitions=port_definitions,
    )


def delete_equipment_type_with_equipments(
    client: SymphonyClient, equipment_type: EquipmentType
) -> None:
    """Delete equipment type with existing equipments.

    :param equipment_type: Existing equipment type name
    :type equipment_type: :class:`~psym.common.data_class.EquipmentType`

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: if equipment_type does not exist

    :return: None

    **Example**

    .. code-block:: python

        equipment_type = client.get_or_create_equipment_type(
            name="Tp-Link T1600G",
            category="Router",
            properties=[
                ("IP", "string", None, True),
            ],
            ports_dict={
                "Port 1": "eth port",
                "port 2": "eth port",
            },
            position_list=[],
        )
        client.delete_equipment_type_with_equipments(equipment_type=equipment_type)
    """
    equipment_type_with_equipments = EquipmentTypeEquipmentQuery.execute(
        client, id=equipment_type.id
    )
    if not equipment_type_with_equipments:
        raise EntityNotFoundError(
            entity=Entity.EquipmentType, entity_id=equipment_type.id
        )
    for equipment in equipment_type_with_equipments.equipments:
        delete_equipment(client, format_to_equipment(equipment_fragment=equipment))

    delete_equipment_type(client=client, equipment_type_id=equipment_type.id)


def delete_equipment_type(client: SymphonyClient, equipment_type_id: str) -> None:
    """This function deletes an equipment type.
    It gets the requested equipment type ID

    :param equipment_type_id: Equipment type ID
    :type equipment_type_id: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Equipment type does not exist

    **Example**

    .. code-block:: python

        client.delete_equipment_type(
            equipment_type_id="12345678"
        )
    """
    equipment_type = get_equipment_type_by_id(
        client=client, equipment_type_id=equipment_type_id
    )
    if equipment_type is None:
        raise EntityNotFoundError(
            entity=Entity.EquipmentType, entity_id=equipment_type_id
        )
    RemoveEquipmentTypeMutation.execute(client, id=equipment_type_id)
    del EQUIPMENT_TYPES[equipment_type.name]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.image_entity import ImageEntity


# fmt: off
QUERY: List[str] = ["""
mutation DeleteImageMutation(
  $entityType: ImageEntity!
  $entityId: ID!
  $id: ID!
) {
  deleteImage(entityType: $entityType, entityId: $entityId, id: $id) {
    id
    fileName
  }
}

"""
]


class DeleteImageMutation:
    @dataclass(frozen=True)
    class DeleteImageMutationData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class File(DataClassJsonMixin):
            id: str
            fileName: str

        deleteImage: File

    # fmt: off
    @classmethod
    def execute(cls, client: Client, entityType: ImageEntity, entityId: str, id: str) -> DeleteImageMutationData.File:
        variables: Dict[str, Any] = {"entityType": entityType, "entityId": entityId, "id": id}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.DeleteImageMutationData.from_dict(response_text)
        return res.deleteImage

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, entityType: ImageEntity, entityId: str, id: str) -> DeleteImageMutationData.File:
        variables: Dict[str, Any] = {"entityType": entityType, "entityId": entityId, "id": id}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.DeleteImageMutationData.from_dict(response_text)
        return res.deleteImage



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.edit_recommendations_input import EditRecommendationsInput


# fmt: off
QUERY: List[str] = ["""
mutation editRecommendations($input: EditRecommendationsInput!) {
  editRecommendations(input: $input) {
		id
    externalID
    resource
    alarmType
    shortDescription
    longDescription
    command
    priority
    status
    runbook
    used
    recommendationsSources{id}
    recommendationsCategory{id}
    userApprobed{id}
    vendor{id}
    
  }
}
"""
]


class editRecommendations:
    @dataclass(frozen=True)
    class editRecommendationsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class Recommendations(DataClassJsonMixin):
            @dataclass(frozen=True)
            class RecommendationsSources(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class RecommendationsCategory(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class User(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Vendor(DataClassJsonMixin):
                id: str

            id: str
            externalID: str
            resource: str
            alarmType: str
            shortDescription: str
            longDescription: str
            command: Optional[str]
            priority: int
            status: bool
            runbook: Optional[str]
            used: Optional[int]
            recommendationsSources: RecommendationsSources
            recommendationsCategory: RecommendationsCategory
            userApprobed: Optional[User]
            vendor: Vendor

        editRecommendations: Recommendations

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: EditRecommendationsInput) -> editRecommendationsData.Recommendations:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editRecommendationsData.from_dict(response_text)
        return res.editRecommendations

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: EditRecommendationsInput) -> editRecommendationsData.Recommendations:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editRecommendationsData.from_dict(response_text)
        return res.editRecommendations



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import List, Mapping, Optional

from psym.client import SymphonyClient
from psym.common.cache import SERVICE_TYPES
from psym.common.data_class import Customer, Link, Service, ServiceEndpoint
from psym.common.data_enum import Entity

from .._utils import PropertyValue, get_graphql_property_inputs
from ..exceptions import EntityNotFoundError
from ..graphql.enum.service_status import ServiceStatus
from ..graphql.input.add_service_endpoint_input import AddServiceEndpointInput
from ..graphql.input.service_create_data import ServiceCreateData
from ..graphql.mutation.add_service import AddServiceMutation
from ..graphql.mutation.add_service_endpoint import AddServiceEndpointMutation
from ..graphql.mutation.add_service_link import AddServiceLinkMutation
from ..graphql.query.service_details import ServiceDetailsQuery
from ..graphql.query.service_endpoints import ServiceEndpointsQuery
from ..graphql.query.service_links import ServiceLinksQuery


def add_service(
    client: SymphonyClient,
    name: str,
    external_id: Optional[str],
    service_type: str,
    customer: Optional[Customer],
    properties_dict: Optional[Mapping[str, PropertyValue]],
) -> Service:
    """This function creates service.

    :param name: Service name
    :type name: str
    :param external_id: Service external ID
    :type external_id: str, optional
    :param service_type: Service type name
    :type service_type: str
    :param customer: Customer object
    :type customer: :class:`~psym.common.data_class.Customer`, optional
    :param properties_dict: Mapping of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Mapping[str, PropertyValue], optional

    :raises:
        FailedOperationException: Internal symphony error

    :return: Service object
    :rtype: :class:`~psym.common.data_class.Service`

    **Example**

    .. code-block:: python

        service = client.add_service(
            name="Room 202 Internet Access",
            external_id="S32325",
            service_type=self.service_type.name,
            customer=None,
            properties_dict={"Address Family": "v4"},
        )
    """
    properties = []
    if properties_dict is not None:
        property_types = SERVICE_TYPES[service_type].property_types
        properties = get_graphql_property_inputs(property_types, properties_dict)
    service_create_data = ServiceCreateData(
        name=name,
        externalId=external_id,
        serviceTypeId=SERVICE_TYPES[service_type].id,
        status=ServiceStatus.PENDING,
        customerId=customer.id if customer is not None else None,
        properties=properties,
        upstreamServiceIds=[],
    )
    result = AddServiceMutation.execute(client, data=service_create_data)
    if customer is not None:
        customer = Customer(
            id=customer.id, name=customer.name, external_id=customer.external_id
        )
    return Service(
        id=result.id,
        name=result.name,
        external_id=result.externalId,
        service_type_name=result.serviceType.name,
        customer=customer,
        properties=result.properties,
    )


def get_service(client: SymphonyClient, id: str) -> Service:
    """This function returns service by ID.

    :param id: Service ID
    :type id: str

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: Service does not exist
        * FailedOperationException: Internal symphony error

    :return: Service object
    :rtype: :class:`~psym.common.data_class.Service`

    **Example**

    .. code-block:: python

        service = client.get_service(id="12345")
    """
    result = ServiceDetailsQuery.execute(client, id=id)
    if result is None:
        raise EntityNotFoundError(entity=Entity.Service, entity_id=id)
    customer_result = result.customer if result.customer is not None else None
    customer: Optional[Customer] = None
    if customer_result is not None:
        customer = Customer(
            id=customer_result.id,
            name=customer_result.name,
            external_id=customer_result.externalId,
        )
    return Service(
        id=result.id,
        name=result.name,
        external_id=result.externalId if result.externalId else None,
        service_type_name=result.serviceType.name,
        customer=customer,
        properties=result.properties,
    )


def get_service_endpoints(
    client: SymphonyClient, service_id: str
) -> List[ServiceEndpoint]:
    """This function returns service endpoints list.

    :param service_id: Service ID
    :type service_id: str

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: Service does not exist
        * FailedOperationException: Internal symphony error

    :return: ServiceEndpoints List
    :rtype: List[ :class:`~psym.common.data_class.ServiceEndpoints` ]

    **Example**

    .. code-block:: python

        endpoints = client.get_service_endpoint_definitions(id="service_id")
    """
    service_data = ServiceEndpointsQuery.execute(client, id=service_id)

    if not service_data:
        raise EntityNotFoundError(entity=Entity.Service, entity_id=service_id)

    return [
        ServiceEndpoint(
            id=endpoint.id,
            equipment_id=endpoint.equipment.id,
            service_id=service_id,
            definition_id=endpoint.definition.id,
        )
        for endpoint in service_data.endpoints
    ]


def add_service_endpoint(
    client: SymphonyClient,
    service: Service,
    equipment_id: str,
    endpoint_definition_id: str,
) -> None:
    """This function adds existing endpoint to existing service.

    :param service: Existing service object
    :type service: :class:`~psym.common.data_class.Service`
    :param equipment_id: Existing equipment ID
    :type equipment_id: str
    :param endpoint_definition_id: Existing endpoint definition ID
    :type endpoint_definition_id: str

    :raises:
        FailedOperationException: Internal symphony error

    :rtype: None

    **Example**

    .. code-block:: python

        service = client.get_service(id="service_id")
        location = client.get_location(location_hirerchy=[("Country", "LS_IND_Prod_Copy")])
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        client.add_service_endpoint(
            service=service,
            equipment_id=equipment.id,
            endpoint_definition_id="endpoint_definition_id,
        )
    """
    endpoint_definition_ids = [
        ed.id for ed in SERVICE_TYPES[service.service_type_name].endpoint_definitions
    ]

    if endpoint_definition_id not in endpoint_definition_ids:
        raise EntityNotFoundError(
            entity=Entity.ServiceEndpointDefinition, entity_id=endpoint_definition_id
        )

    AddServiceEndpointMutation.execute(
        client,
        input=AddServiceEndpointInput(
            id=service.id, definition=endpoint_definition_id, equipmentID=equipment_id
        ),
    )


def get_service_links(client: SymphonyClient, service_id: str) -> List[Link]:
    """This function returns list of Links.

    :param service_id: Service ID
    :type service_id: str

    :raises:
        * :class:`~psym.exceptions.EntityNotFoundError`: Service does not exist
        * FailedOperationException: Internal symphony error

    :return: Links List
    :rtype: List[ :class:`~psym.common.data_class.Link` ]

    **Example**

    .. code-block:: python

        links = client.get_service_links(id="service_id")
    """
    service_data = ServiceLinksQuery.execute(client, id=service_id)

    if not service_data:
        raise EntityNotFoundError(entity=Entity.Service, entity_id=service_id)

    return [
        Link(
            id=link.id,
            properties=link.properties,
            service_ids=[s.id for s in link.services],
        )
        for link in service_data.links
    ]


def add_service_link(client: SymphonyClient, service_id: str, link_id: str) -> None:
    """This function adds existing link to existing service.

    :param service_id: Existing service ID
    :type service_id: str
    :param link_id: Existing link ID
    :type link_id: str

    :raises:
        FailedOperationException: Internal symphony error

    :rtype: None

    **Example**

    .. code-block:: python

        client.add_service_link(service_id=service.id, link_id=link.id)
    """
    AddServiceLinkMutation.execute(client, id=service_id, linkId=link_id)



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, List, Optional, Sequence, Union

from psym.client import SymphonyClient
from psym.common.cache import (
    EQUIPMENT_TYPES,
    LOCATION_TYPES,
    PORT_TYPES,
    SERVICE_TYPES,
    Cache,
)
from psym.common.data_class import (
    EquipmentPortType,
    EquipmentType,
    LocationType,
    PropertyDefinition,
    ServiceType,
)
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_property_type_input

from ..exceptions import EntityNotFoundError
from ..graphql.input.property_type_input import PropertyTypeInput


def get_property_types(
    client: SymphonyClient, entity_type: Entity, entity_name: str
) -> Sequence[PropertyDefinition]:
    """Get property types on specific entity.

    :param entity_type: Existing entity type
    :type entity_type: :class:`~psym.common.data_enum.Entity`
    :param entity_name: Existing entity name
    :type entity_name: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: `entity_type` does not found or does not have property types

    :return: Sequence[PropertyDefinition]
    :rtype: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]

    **Example**

    .. code-block:: python

        property_type = client.get_property_types(
            entity_type=Entity.EquipmentType,
            entity_name="Card",
        )
    """

    caches: Dict[
        Entity,
        Union[
            Cache[LocationType],
            Cache[EquipmentType],
            Cache[ServiceType],
            Cache[EquipmentPortType],
        ],
    ] = {
        Entity.LocationType: LOCATION_TYPES,
        Entity.EquipmentType: EQUIPMENT_TYPES,
        Entity.ServiceType: SERVICE_TYPES,
        Entity.EquipmentPortType: PORT_TYPES,
    }

    existing_entity_types: Optional[
        Union[
            Cache[LocationType],
            Cache[EquipmentType],
            Cache[ServiceType],
            Cache[EquipmentPortType],
        ],
    ] = caches.get(entity_type, None)

    if existing_entity_types is None:
        raise EntityNotFoundError(entity=entity_type)

    existing_entity_type = None
    if entity_name in existing_entity_types:
        existing_entity_type = existing_entity_types[entity_name]

    if existing_entity_type is None:
        raise EntityNotFoundError(entity=entity_type, entity_name=entity_name)

    return existing_entity_type.property_types


def get_property_type(
    client: SymphonyClient, entity_type: Entity, entity_name: str, property_type_id: str
) -> PropertyDefinition:
    """Get property type on specific entity.

    :param entity_type: Existing entity type
    :type entity_type: :class:`~psym.common.data_enum.Entity`
    :param entity_name: Existing entity name
    :type entity_name: str
    :param property_type_id: Existing property type ID
    :type property_type_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Property type with id=`property_type_id` does not found

    :return: PropertyDefinition object
    :rtype: :class:`~psym.common.data_class.PropertyDefinition`

    **Example**

    .. code-block:: python

        property_type = client.get_property_type(
            entity_type=Entity.EquipmentType,
            entity_name="Card",
            property_type_id="12345",
        )
    """
    property_types = get_property_types(
        client=client, entity_type=entity_type, entity_name=entity_name
    )
    for property_type in property_types:
        if property_type.id == property_type_id:
            return property_type

    raise EntityNotFoundError(entity=Entity.PropertyType, entity_id=property_type_id)


def get_property_type_id(
    client: SymphonyClient,
    entity_type: Entity,
    entity_name: str,
    property_type_name: str,
) -> str:
    """Get property type ID on specific entity.

    :param entity_type: Existing entity type
    :type entity_type: :class:`~psym.common.data_enum.Entity`
    :param entity_name: Existing entity name
    :type entity_name: str
    :param property_type_name: Existing property type name
    :type property_type_name: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Property type with name=`property_type_name` does not found

    :return: property type ID
    :rtype: str

    **Example**

    .. code-block:: python

        property_type = client.get_property_type_id(
            entity_type=Entity.EquipmentType,
            entity_name="Card",
            property_type_name="IP",
        )
    """
    property_types = get_property_types(
        client=client, entity_type=entity_type, entity_name=entity_name
    )
    for property_type in property_types:
        if (
            property_type.property_name == property_type_name
            and property_type.id is not None
        ):
            return property_type.id

    raise EntityNotFoundError(
        entity=Entity.PropertyType, entity_name=property_type_name
    )


def get_property_type_by_external_id(
    client: SymphonyClient,
    entity_type: Entity,
    entity_name: str,
    property_type_external_id: str,
) -> PropertyDefinition:
    """Get property type by external ID on specific entity.

    :param entity_type: Existing entity type
    :type entity_type: :class:`~psym.common.data_enum.Entity`
    :param entity_name: Existing entity name
    :type entity_name: str
    :param property_type_external_id: Existing property type external ID
    :type property_type_external_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Property type with external_id=`property_type_external_id` does not found

    :return: PropertyDefinition object
    :rtype: :class:`~psym.common.data_class.PropertyDefinition`

    **Example**

    .. code-block:: python

        property_type = client.get_property_type_by_external_id(
            entity_type=Entity.EquipmentType,
            entity_name="Card",
            property_type_external_id="12345",
        )
    """
    property_types = get_property_types(
        client=client, entity_type=entity_type, entity_name=entity_name
    )
    for property_type in property_types:
        if property_type.external_id == property_type_external_id:
            return property_type

    raise EntityNotFoundError(
        entity=Entity.PropertyType, msg=f"<external_id: {property_type_external_id}>"
    )


def edit_property_type(
    client: SymphonyClient,
    entity_type: Entity,
    entity_name: str,
    property_type_id: str,
    new_property_definition: PropertyDefinition,
) -> List[PropertyTypeInput]:
    """Edit specific property type on specific entity. `entity_type` - ["LocationType", "EquipmentType", "ServiceType", "EquipmentPortType"]

    :param entity_type: Existing entity type
    :type entity_type: :class:`~psym.common.data_enum.Entity`
    :param entity_name: Existing entity name
    :type entity_name: str
    :param property_type_id: Existing property type ID
    :type property_type_id: str
    :param new_property_definition: New property definition
    :type new_property_definition: :class:`~psym.common.data_class.PropertyDefinition`

    :class:`~psym.exceptions.EntityNotFoundError`: Property type with id=`property_type_id` does not found

    :return: PropertyTypeInputs List
    :rtype: List[ :class:`~psym.graphql.input.property_type.PropertyTypeInput` ]

    **Example**

    .. code-block:: python

        property_types = client.edit_property_type(
            entity_type=Entity.EquipmentType,
            entity_name="Card",
            property_type_id="12345",
            property_definition=PropertyDefinition(
                property_name="new_name",
                property_kind=PropertyKind.string,
                default_raw_value=None,
                is_fixed=False,
                external_id="ex_12345",
            ),
        )
    """
    property_types = get_property_types(
        client=client, entity_type=entity_type, entity_name=entity_name
    )
    edited_property_types = []

    for property_type in property_types:
        if property_type.id == property_type_id:
            property_type_input = format_to_property_type_input(
                new_property_definition,
                is_new=False,
                property_type_id=property_type_id,
            )
        else:
            property_type_input = format_to_property_type_input(
                property_type, is_new=False
            )
        edited_property_types.append(property_type_input)

    return edited_property_types



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ImageEntity(Enum):
    LOCATION = "LOCATION"
    WORK_ORDER = "WORK_ORDER"
    SITE_SURVEY = "SITE_SURVEY"
    EQUIPMENT = "EQUIPMENT"
    USER = "USER"
    CHECKLIST_ITEM = "CHECKLIST_ITEM"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ImageEntity":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

# fmt: off
QUERY: List[str] = ["""
fragment PropertyFragment on Property {
  id
  propertyType {
    id
    externalId
    name
  }
  stringValue
  intValue
  floatValue
  booleanValue
  latitudeValue
  longitudeValue
  rangeFromValue
  rangeToValue
}

"""]

@dataclass(frozen=True)
class PropertyFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class PropertyType(DataClassJsonMixin):
        id: str
        externalId: Optional[str]
        name: str

    id: str
    propertyType: PropertyType
    stringValue: Optional[str]
    intValue: Optional[int]
    floatValue: Optional[Number]
    booleanValue: Optional[bool]
    latitudeValue: Optional[Number]
    longitudeValue: Optional[Number]
    rangeFromValue: Optional[Number]
    rangeToValue: Optional[Number]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.check_list_item_enum_selection_mode import CheckListItemEnumSelectionMode
from ..enum.check_list_item_type import CheckListItemType


@dataclass(frozen=True)
class CheckListDefinitionInput(DataClassJsonMixin):
    title: str
    type: CheckListItemType = _field(metadata=enum_field_metadata(CheckListItemType))
    id: Optional[str] = None
    index: Optional[int] = None
    isMandatory: Optional[bool] = None
    enumValues: Optional[str] = None
    enumSelectionMode: Optional[CheckListItemEnumSelectionMode] = None
    helpText: Optional[str] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import organization
from psym.common.data_enum import Entity
from psym.exceptions import EntityNotFoundError

from ..graphql.input.edit_organization_input import EditOrganizationInput
from ..graphql.input.add_organization_input import AddOrganizationInput
from ..graphql.mutation.add_organization import addOrganization
from ..graphql.mutation.edit_organization import editOrganization
from ..graphql.mutation.remove_organization import removeOrganization
from ..graphql.query.organizations import organizations
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_organization(
    client: SymphonyClient, name: str, description: str
) -> organization:
    """This function adds Organization.

    :param name: Organization name
    :type name: str

    :return: organization object
    :rtype: :class:`~psym.common.data_class.organization`

    **Example 1**

    .. code-block:: python

        new_organization = client.add_organization(
            name="organization_1",
            description="organization"

        )
    """
    organization_input = AddOrganizationInput(name=name, description=description)
    if (len(name) != 0):
        result = addOrganization.execute(client, input=organization_input)
        return organization(name=result.name, id=result.id, description=result.description)
    else:
        print ("you cant leave the name parameter empty")

def edit_organization(
    client: SymphonyClient,
    organization: organization,
    new_name: Optional[str] = None,
    new_description: Optional[str] = None,
) -> None:
    """This function edits Organization.

    :param organization: Organization entity
    :type name: str
    :param new_name: Organization name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.organization`


    **Example 1**

    .. code-block:: python

        new_organization = client.edit_organization(
            organization=organization,
            new_name="new_name",
            new_description="organization"

        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_description is not None:
        params.update({"_name_": new_description})
    if new_name is not None:
        editOrganization.execute(client, input=EditOrganizationInput(id=organization.id, name=new_name, description=new_description))

def get_organizations(client: SymphonyClient) -> Iterator[organization]:
    """ This funtion Get Organizations


    :return: organization object
    :rtype: Iterator[ :class:`~psym.common.data_class.organization` ]

    **Example**

    .. code-block:: python

        organizationes = client.get_organizations()
        for organization in organizations:
            print(organization.name)
    """
    organizations_ = organizations.execute(client, first=PAGINATION_STEP)
    edges = organizations_.edges if organizations_ else []
    while organizations_ is not None and organizations_.pageInfo.hasNextPage:
        organizations_ = organizations.execute(
            client, after=organizations_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if organizations_ is not None:
            edges.extend(organizations_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield organization(
                id=node.id,
                name=node.name,
                description=node.description
            )

def get_organization_by_name(client: SymphonyClient, name: str) -> organization:
    """ This funtion Get Organizations


    :return: organization object
    :rtype: Iterator[ :class:`~psym.common.data_class.organization` ]

    **Example**

    .. code-block:: python

        organization_name:client.get_organization_by_name(name="test")
        print(organization_name)
    """
    organization_name = get_organizations(client=client)
    for organization_names in organization_name:
        if organization_names.name == name:
            return organization_names
    raise EntityNotFoundError(entity=Entity.organization, entity_name=name)


def remove_organization(client: SymphonyClient, id: str) -> None:
    """This function delete Organization.

    :param name: Organization name
    :type name: :class:`~psym.common.data_class.organization`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_organization(id=123456789)
    """
    removeOrganization.execute(client, id=id)





#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.entry_point_input import EntryPointInput
from ..input.exit_point_input import ExitPointInput


@dataclass(frozen=True)
class ConnectorInput(DataClassJsonMixin):
    sourceBlockCid: str
    targetBlockCid: str
    sourcePoint: Optional[ExitPointInput] = None
    targetPoint: Optional[EntryPointInput] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class UserRole(Enum):
    USER = "USER"
    ADMIN = "ADMIN"
    OWNER = "OWNER"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "UserRole":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class VariableExpressionType(Enum):
    VariableDefinition = "VariableDefinition"
    PropertyTypeDefinition = "PropertyTypeDefinition"
    DecisionDefinition = "DecisionDefinition"
    ChekListItemDefinition = "ChekListItemDefinition"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "VariableExpressionType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class EquipmentFilterType(Enum):
    EQUIP_INST_NAME = "EQUIP_INST_NAME"
    EQUIP_INST_EXTERNAL_ID = "EQUIP_INST_EXTERNAL_ID"
    PROPERTY = "PROPERTY"
    LOCATION_INST = "LOCATION_INST"
    LOCATION_INST_EXTERNAL_ID = "LOCATION_INST_EXTERNAL_ID"
    EQUIPMENT_TYPE = "EQUIPMENT_TYPE"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "EquipmentFilterType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.formula_order_field import FormulaOrderField
from ..enum.order_direction import OrderDirection


@dataclass(frozen=True)
class FormulaOrder(DataClassJsonMixin):
    direction: OrderDirection = _field(metadata=enum_field_metadata(OrderDirection))
    field: Optional[FormulaOrderField] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.variable_value_input import VariableValueInput


@dataclass(frozen=True)
class StartFlowInput(DataClassJsonMixin):
    flowID: str
    bssCode: str
    startDate: datetime = _field(metadata=config(encoder=custom_scalars["Time"].encoder, decoder=custom_scalars["Time"].decoder, mm_field=custom_scalars["Time"].mm_field))
    params: List[VariableValueInput]



#!/usr/bin/env python3


import glob
import os
from datetime import datetime
from typing import Generator, Optional

import filetype
from psym.client import SymphonyClient
from psym.common.data_class import Document, Location

from ..graphql.enum.image_entity import ImageEntity
from ..graphql.input.add_image_input import AddImageInput
from ..graphql.mutation.add_image import AddImageMutation
from ..graphql.mutation.delete_image import DeleteImageMutation


def list_dir(directory_path: str) -> Generator[str, None, None]:
    files = list(glob.glob(os.path.join(directory_path, "**/**"), recursive=True))
    for file_path in set(files):
        if os.path.isfile(file_path):
            yield file_path


def add_file(
    client: SymphonyClient,
    local_file_path: str,
    entity_type: str,
    entity_id: str,
    category: Optional[str] = None,
    document_category_id: Optional[str] = None,
) -> None:
    """This function adds file to an entity of a given type.

    :param local_file_path: Local system path to the file
    :type local_file_path: str
    :param entity_type: One of existing options ["LOCATION", "WORK_ORDER", "SITE_SURVEY", "EQUIPMENT"]
    :type entity_type: str
    :param category: File category name
    :type category: str, optional
    :param document_category_id: Document category id
    :type document_category_id: int, optional

    :raises:
        FailedOperationException: on operation failure

    :return: None

    **Example**

    .. code-block:: python

        document_category = PsymClient.get_document_category_by_names(
            client=client_symp,
            location_type_name="Country",
            doc_category_name="Correo de notificación"
        )
        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        client.add_file(
            local_file_path="./document.pdf",
            entity_type="LOCATION",
            entity_id=location.id,
            document_category_id=document_category.id
        )
    """
    entity = {
        "LOCATION": ImageEntity.LOCATION,
        "WORK_ORDER": ImageEntity.WORK_ORDER,
        "SITE_SURVEY": ImageEntity.SITE_SURVEY,
        "EQUIPMENT": ImageEntity.EQUIPMENT,
    }.get(entity_type, ImageEntity.LOCATION)
    add_image(
        client, local_file_path, entity, entity_id, category, document_category_id
    )


def add_files(
    client: SymphonyClient,
    local_directory_path: str,
    entity_type: str,
    entity_id: str,
    category: Optional[str] = None,
    document_category_id: Optional[int] = None,
) -> None:
    """This function adds all files located in folder to an entity of a given type.

    :param local_directory_path: Local system path to the directory
    :type local_directory_path: str
    :param entity_type: One of existing options ["LOCATION", "WORK_ORDER", "SITE_SURVEY", "EQUIPMENT"]
    :type entity_type: str
    :param category: File category name
    :type category: str, optional
    :param document_category_id: Document category id
    :type document_category_id: int, optional

    :raises:
        FailedOperationException: on operation failure

    :return: None

    **Example**

    .. code-block:: python

        document_category = PsymClient.get_document_category_by_names(
            client=client_symp,
            location_type_name="Country",
            doc_category_name="Correo de notificación"
        )
        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        client.add_files(
            local_directory_path="./documents_folder/",
            entity_type="LOCATION",
            entity_id=location.id,
            document_category_id=document_category.id
        )
    """
    for file in list_dir(local_directory_path):
        add_file(client, file, entity_type, entity_id, category, document_category_id)


def add_location_image(
    client: SymphonyClient,
    local_file_path: str,
    location: Location,
    document_category_id: Optional[str] = None,
) -> None:
    """This function adds image to existing location.

    :param local_file_path: Local system path to the file
    :type local_file_path: str
    :param location: Existing location object, could be retrieved from

        * :meth:`~psym.api.location.get_location`
        * :meth:`~psym.api.location.add_location`
    :param document_category_id: Document category id
    :type document_category_id: int, optional

    :type location: :class:`~psym.common.data_class.Location`

    :raises:
        FailedOperationException: on operation failure

    :return: None

    **Example**

    .. code-block:: python
        document_category = PsymClient.get_document_category_by_names(
            location_type_name="Country",
            doc_category_name="Correo de notificación",
        )
        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        client.add_location_image(
            local_file_path="./document.pdf",
            location=location,
            document_category_id=document_category.id
        )
    """
    add_image(
        client,
        local_file_path,
        ImageEntity.LOCATION,
        location.id,
        documentCategoryId=document_category_id,
    )


def delete_document(client: SymphonyClient, document: Document) -> None:
    """This function deletes existing document.

    :param document: Document object
    :type document: :class:`~psym.common.data_class.Document`

    :raises:
        FailedOperationException: on operation failure

    :return: None

    **Example**

    .. code-block:: python

        client.delete_document(document=document)
    """
    delete_image(client, document.parent_entity, document.parent_id, document.id)


def add_image(
    client: SymphonyClient,
    local_file_path: str,
    entity_type: ImageEntity,
    entity_id: str,
    category: Optional[str] = None,
    documentCategoryId: Optional[str] = None,
) -> None:
    file_type = filetype.guess(local_file_path)
    file_type = file_type.MIME if file_type is not None else ""
    img_key = client.store_file(local_file_path, file_type, False)
    file_size = os.path.getsize(local_file_path)

    AddImageMutation.execute(
        client,
        AddImageInput(
            entityType=entity_type,
            entityId=entity_id,
            imgKey=img_key,
            fileName=os.path.basename(local_file_path),
            fileSize=file_size,
            modified=datetime.utcnow(),
            contentType=file_type,
            category=category,
            documentCategoryId=documentCategoryId,
        ),
    )


def delete_image(
    client: SymphonyClient, entity_type: ImageEntity, entity_id: str, image_id: str
) -> None:
    DeleteImageMutation.execute(
        client, entityType=entity_type, entityId=entity_id, id=image_id
    )



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

__version__ = "2.5.31"

SCHEMA_FILE_NAME = "survey_schema.json"
SIMPLE_QUESTION_TYPE_TO_REQUIRED_PROPERTY_NAME = {
    "DATE": "dateData",
    "BOOL": "boolData",
    "EMAIL": "emailData",
    "TEXT": "textData",
    "FLOAT": "floatData",
    "INTEGER": "intData",
    "PHONE": "phoneData",
}

EQUIPMENTS_TO_SEARCH = 10
LOCATIONS_TO_SEARCH = 5
PAGINATION_STEP = 1000
USER_ROLE = 0
SUPERUSER_ROLE = 3

GET_FEATURES_URL = "/features/async/"
SET_FEATURE_URL = "/features/async/{}"